########################################################################
# updateDefectAggregate.pl
#
# This file's purpose is to populate the ChangeRequest_aggregate table
# with the number of open BP1 defects for each product.  
#
########################################################################

use PjCLib;
use Win32::OLE;
require CQPerlExt;

# Define here the strings you want to use to reflect the project status
use constant GREEN => "1-Green";
use constant YELLOW => "2-Yellow";
use constant RED => "3-Red";


################
# main
################

# global variables
$currentDate; 
$currentTime;

getCurrentDateTime();
print "Start Time = $currentDate $currentTime \n";

$session = logon();

myMain();

getCurrentDateTime();
print "End Time = $currentDate $currentTime \n";

exit;


sub myMain {

    print "Updating defect aggregate table. Please wait...\n";

    PjCLib->initializePjCLibrary($session, PjCLib->BOOLEAN_TRUE);
    initialize();

    deleteRecordsFromAggregateTable($CR_AGGREGATE_MEASURE_TABLE);

    insertAggregateMeasures('ClassicsPOS');
    insertAggregateMeasures('Webshop');


    print "Done updating defect aggregate table.\n";

}


sub initialize {

    # Get physical table names. These names will be used in the query definitions.
    $CR_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("ChangeRequest_Measure");
    $CR_AGGREGATE_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("ChangeRequestAggregate_Measure");
    $PLANVALUES_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("PlanValues_measure");
    $COLLECTIONDATE_TABLE = PjCLib->getPhysicalTableDbName("collectiondate");
    $CALENDARDATE_TABLE = PjCLib->getPhysicalTableDbName("calendardate");
    $PRODUCT_DIM = PjCLib->getPhysicalTableDbName("Product_Dim");
    $PROJECT_DIM = PjCLib->getPhysicalTableDbName("project_dim");
    $PROJECTSTATUS_DIM = PjCLib->getPhysicalTableDbName("ProjectStatus_Dim");
    $PRIORITY_DIM = PjCLib->getPhysicalTableDbName("priority_dim");
    $STATE_DIM = PjCLib->getPhysicalTableDbName("state_dim");
    

    # Get physical field names for the dimension tables. These names will be used in the query definitions.
    $COLLECTIONDATE_NAME = PjCLib->getPhysicalFieldDbName("collectiondate", "collectiondate_name");
    $CALENDARDATE_NAME = PjCLib->getPhysicalFieldDbName("calendardate", "calendardate_name");
    $PRODUCT_NAME = PjCLib->getPhysicalFieldDbName("Product_Dim", "Product");
    $PROJECT_NAME = PjCLib->getPhysicalFieldDbName("project_dim", "project");
    $PROJECTSTATUS_NAME = PjCLib->getPhysicalFieldDbName("ProjectStatus_Dim", "Project Status");
    $PRIORITY_NAME = PjCLib->getPhysicalFieldDbName("priority_dim", "priority");
    $STATE_NAME = PjCLib->getPhysicalFieldDbName("state_dim", "state");

    # Get physical field names for the Change Request Measure table. These names will be used in the query definitions.
    $PRODUCT_FK = PjCLib->getPhysicalFieldDbName("ChangeRequest_Measure", "product_fk");
    $PROJECT_FK = PjCLib->getPhysicalFieldDbName("ChangeRequest_Measure", "project_fk");
    $PRIORITY_FK = PjCLib->getPhysicalFieldDbName("ChangeRequest_Measure", "priority_fk");
    $STATE_FK = PjCLib->getPhysicalFieldDbName("ChangeRequest_Measure", "state_fk");
    $COLLECTIONDATE_FK = PjCLib->getPhysicalFieldDbName("ChangeRequest_Measure", "collectiondate_fk");

    # Get physical field names for the aggregate table. These names will be used in the query definitions.
    $CR_AGGR_COLLECTIONDATE_FK = PjCLib->getPhysicalFieldDbName("ChangeRequestAggregate_Measure", "collectiondate_fk");
    $CR_AGGR_PRODUCT_FK = PjCLib->getPhysicalFieldDbName("ChangeRequestAggregate_Measure", "Product_fk");
    $CR_AGGR_PROJECT_FK = PjCLib->getPhysicalFieldDbName("ChangeRequestAggregate_Measure", "Project_fk");
    $CR_AGGR_PROJECTSTATUS_FK = PjCLib->getPhysicalFieldDbName("ChangeRequestAggregate_Measure", "ProjectStatus_fk");
    $CR_AGGR_OPEN_CR = PjCLib->getPhysicalFieldDbName("ChangeRequestAggregate_Measure", "Open High Priority Change Requests");    
    $CR_AGGR_GLIDEPATH = PjCLib->getPhysicalFieldDbName("ChangeRequestAggregate_Measure", "Glide Path");
    $CR_AGGR_CR_TO_VERIFY = PjCLib->getPhysicalFieldDbName("ChangeRequestAggregate_Measure", "Change Requests To Verify");

    # Get physical field names for the plan value table.
    $PLANNED_VALUE = PjCLib->getPhysicalFieldDbName("PlanValues_measure", "Planned High Priority ChangeRequests");
    $PROJECT_VALUE = PjCLib->getPhysicalFieldDbName("PlanValues_measure", "Project_fk");
    $CALENDARDATE_FK = PjCLib->getPhysicalFieldDbName("PlanValues_measure", "calendardate_fk");

}


sub logon {

    #
    # Use the internal DashboardSystem login to get access to the Dashboard database
    #
    my $session = Win32::OLE->new('CLEARQUEST.SESSION');
    my $session = CQPerlExt::CQSession_Build();  
    $session->UserLogon("dashboardsystem", "dashboardsystem", "tstd1", "dishigaki-w2kp");
    return $session;
}


# get date and set the global variables $currentDate and $currentTime
sub getCurrentDateTime {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

    $mon = $mon + 1;
    $year = $year + 1900;

    if ($mday<10) {$mday="0$mday"; }  # add leading 0
    if ($mon<10) {$mon="0$mon"; }     # add leading 0

    # set the global variable $currentDate
#    $currentDate="$year-$mon-$mday";

    $currentDate="2003-01-24";


    # set the global variable $currentTime
    $currentTime = $hour . ":" . $min . ":" . $sec;

}


sub insertAggregateMeasures {
    my ($project) = @_;

    $OpenCRCount = 0;
    $CRToVerifyCount = 0;
    $Glidepath = 0;
 
    $OpenCRCount = calculateOpenChangeRequests($project);

    $CRToVerifyCount = calculateChangeRequestsToVerify($project);

    $Glidepath = calculateGlidepath($OpenCRCount,
                      		    $project,
	       		    	    $currentDate);

    $status = getProjectStatus($Glidepath);

    updateAggregateTable($OpenCRCount,
			 $CRToVerifyCount,
			 $Glidepath,
                         $project,
			 $status,
			 $currentDate);

}


#
# calculate Number of Open High Priority Change Requests
#

sub calculateOpenChangeRequests {
    my (
        $project,
       ) = @_;

    # Returns the count of open change requests for the specified project with the following priorities:
    #

    my $NumberOpenCR = 0;

    my $queryStr = "SELECT count(*) FROM " . $CR_MEASURE_TABLE . " WHERE " .
                   $COLLECTIONDATE_FK . " = (SELECT dbid FROM " . $COLLECTIONDATE_TABLE . " WHERE " . 
		   $COLLECTIONDATE_NAME . " = '" . $currentDate . "') AND " .
	           $PROJECT_FK . " = (SELECT dbid FROM " . $PROJECT_DIM . " WHERE " . $PROJECT_NAME . " = '" . $project . "')" .
	           " AND " . $PRIORITY_FK . " IN (SELECT dbid FROM " . $PRIORITY_DIM . " WHERE " . $PRIORITY_NAME . " IN" .
		   " ('High', '1-Resolve Immediately', '2-Give High Attention')) AND " . 
		   $STATE_FK . " IN (SELECT dbid FROM " . $STATE_DIM . " WHERE " . $STATE_NAME . " IN ('Opened','Assigned','Resolved'))" ;

    my $resultSet = $session->BuildSQLQuery($queryStr);

    # Execute the query
    $resultSet->Execute();

    # Move to the beginning of the first row (unique row since we returned a count)
    $resultSet->MoveNext();

    # Get the value stored in the result set (which is the result of the select statement)
    $NumberOpenCR = $resultSet->GetColumnValue(1);

    return ($NumberOpenCR);
}


#
# calculate Number of defects to verify
#

sub calculateChangeRequestsToVerify {
    my (
        $project,
       ) = @_;

    # Returns the count of defects to verify for the specified project

    my $NumberCRToVerify = 0;
    my $queryStr = "SELECT count(*) FROM " . $CR_MEASURE_TABLE . " WHERE " .
                   $COLLECTIONDATE_FK . " = (SELECT dbid FROM " . $COLLECTIONDATE_TABLE . " WHERE " . 
		   $COLLECTIONDATE_NAME . " = '" . $currentDate . "') AND " .
	           $PROJECT_FK . " = (SELECT dbid FROM " . $PROJECT_DIM . " WHERE " . $PROJECT_NAME . " = '" . $project . "')" .
	           " AND " . $STATE_FK . " IN (SELECT dbid FROM " . $STATE_DIM . " WHERE " . $STATE_NAME . " = 'Resolved')" ;

    my $resultSet = $session->BuildSQLQuery($queryStr);

    # Execute the query
    $resultSet->Execute();

    # Move to the beginning of the first row (unique row since we returned a count)
    $resultSet->MoveNext();

    # Get the value stored in the result set (which is the result of the select statement)
    $NumberCRToVerify = $resultSet->GetColumnValue(1);

    return ($NumberCRToVerify);
}


#
# calculate Glidepath
#

sub calculateGlidepath {
    my (
        $actualValue,
        $project,
        $dateValue,
       ) = @_;

    my $glidepathValue = 0;

    my $queryStr = "SELECT " . $PLANNED_VALUE . " FROM " . $PLANVALUES_MEASURE_TABLE . " WHERE " .
                   $CALENDARDATE_FK . " = (SELECT dbid FROM " . $CALENDARDATE_TABLE . " WHERE " . 
		   $CALENDARDATE_NAME . " = '" . $currentDate . "') AND " .
	           $PROJECT_VALUE . " = (SELECT dbid FROM " . $PROJECT_DIM . " WHERE " . $PROJECT_NAME . " = '" . $project . "')";

    my $resultSet = $session->BuildSQLQuery($queryStr);

    # Execute the query
    $resultSet->Execute();

    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       # there is a planned value for that day
       # Get the value stored in the result set (which is the result of the select statement)
        
       my $planValue = $resultSet->GetColumnValue(1);

       if ($planValue != 0) {
           $glidepathValue = (($actualValue/$planValue) - 1) * 100;
	   # limit precision to 2 digits
	   my $asStr = sprintf ("%.2f", $glidepathValue);
           $glidepathValue = $asStr;
       }
    }
    else {
       # there is no plan value for that day so we can't calculate the glidepath
       # no action needed since glidepathValue already set to 0
    }
    return ($glidepathValue);
}



#
# Determine Project Status by looking at the Glide path value
#
sub getProjectStatus {
    my (
        $glidePathValue,
       ) = @_;

    my $statusColor = 'No Color';
    print "glidePathValue = '$glidePathValue'\n";


    if ($glidePathValue <= 0.00) {
	$statusColor = GREEN;
    }
    elsif ($glidePathValue <= 50.00) {
        $statusColor = YELLOW;
    }
    else {   #will be.... if ($glidePathValue >50) {
        $statusColor = RED;
    }
    print "statusColor = $statusColor\n";
    return ($statusColor);

}



#
# Insert values into the aggregate table
#

sub updateAggregateTable {
    my (
        $OpenCRCount,
        $CRToVerifyCount,
	$glidepathValue,
        $project,
	$status,
        $dateValue,
       ) = @_;

    my $entity;

    # do an INSERT $CR_AGGREGATE_MEASURE_TABLE
    #print "INSERT aggregate record into $CR_AGGREGATE_MEASURE_TABLE\n";
    $entity = $session->BuildEntity($CR_AGGREGATE_MEASURE_TABLE);
    print "BuildEntity($CR_AGGREGATE_MEASURE_TABLE) succeeded\n";
 
    print "Set $CR_AGGR_COLLECTIONDATE_FK field to $dateValue\n";
    $entity->SetFieldValue($CR_AGGR_COLLECTIONDATE_FK,$dateValue);   

    print "Set $CR_AGGR_PROJECT_FK field to $project\n";
    $entity->SetFieldValue($CR_AGGR_PROJECT_FK,$project);

    print "Set $CR_AGGR_PROJECTSTATUS_FK field to $status\n";
    $entity->SetFieldValue($CR_AGGR_PROJECTSTATUS_FK,$status);

    print "Set $CR_AGGR_OPEN_CR field to $OpenCRCount\n";
    $entity->SetFieldValue($CR_AGGR_OPEN_CR,$OpenCRCount);

    print "Set $CR_AGGR_CR_TO_VERIFY field to $CRToVerifyCount\n";
    $entity->SetFieldValue($CR_AGGR_CR_TO_VERIFY,$CRToVerifyCount);

    print "Set $CR_AGGR_GLIDEPATH field to $glidepathValue\n";
    $entity->SetFieldValue($CR_AGGR_GLIDEPATH,$glidepathValue);

    # commit the record to the database if validation returns no errors
    $status = $entity->Validate();
    if ($status eq "") {
        $entity->Commit();
        printf("Aggregate Measure Table updated\n");
    }
    else {
        $entity->Revert();
	print "Status = $status";
        printf("Aggregate Measure Table not updated\n. Validate action failed.\n");
    }
}



sub deleteRecordsFromAggregateTable () {

    ($tableName) = @_;

    print "tableName: $tableName\n";

    # Declare local variables used by subroutine.

    my($queryStr);           # String used to query the CQ database
    my($resultSet); 	     # Results from executing a SQL query of $tableName
    my($dbid);               # dbid of an existing record in the $tableName
    my($entity);             # Record to delete from $tableName
    my($retval);             # Return value after deleting record from $tableName

    # Determine if the aggregate table already contains records

    $queryStr = "SELECT dbid FROM $tableName WHERE dbid != 0";
       
    $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {

	# Delete any existing records from the aggregate table.

	$dbid = $resultSet->GetColumnValue(1);
	$entity = $session->GetEntityByDbId("$tableName", $dbid);
	$retval = $session->DeleteEntity($entity, "delete");
	#print "retval after deleting record from $tableName: $ retval\n";
   }
}



